"""
Module for graphing
"""

import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
import scipy

def hist_distributivity_graph(N, RESOLUTION, seed, data, name):
    """
    graphs histograms
    see values function in main module
    """
    plt.clf()
    for key in data:
        print("building " + str(name) + " histogram...")
        xlabel = "Value"
        ylabel = "Count"
       
        ## Fitted Normal for comparaison 
        mu, std = scipy.stats.norm.fit(data[key])
        #mean = np.mean(data[key]) 
        #x_pdf = np.linspace(0.1,mean+0.1,1000)
        #y_pdf = scipy.stats.norm.pdf(x_pdf, mu, std)
        
        title_name = name + "\n N = " + str(N) + "\n seed = " + str(seed)
        if seed == 0:
            title_name = name + "\n N = " + str(N)

        graph = sns.displot(data[key], kde=True, bins=RESOLUTION
                            , stat="density")
        graph.set(title=title_name)
        graph.set_axis_labels(xlabel, ylabel)

        #plt.plot(x_pdf, y_pdf, c='r', label="Fitted normal distrib."
        #         , linewidth=1, linestyle='dotted')
        plt.legend(loc="lower right")

        graph.savefig(name + "_hist.png", bbox_inches="tight", dpi=50)

    return

def kde_2distributivity_graph(N, RESOLUTION, seed, data, cov, name):
    """
    graphs histograms
    see values function in main module
    """
    plt.clf()
    
    title_name = name + "\n N = " + str(N) + "\n seed = " + str(seed)
    if seed == 0:
        title_name = name + "\n N = " + str(N)
    title_name = title_name + "\n covariance = \n" + str(cov)
    print("building " + str(name) + " 2D kdeplot...")
    
    cmap = sns.cubehelix_palette(start=0, light=1, as_cmap=True)
    
    x, y = data[0], data[1]
    print("graphing with "+str(N)+" values...")
    graph = sns.kdeplot(x=x, y=y, fill=True, levels=10, cmap=cmap)
    graph.set(title=title_name)
    print("saving picture...")
    plt.savefig(name + "_2D.png", bbox_inches="tight", dpi=500)
    return


def ecdf_graph(N, RESOLUTION, seed, data):
    """
    graphs histograms
    see values function in main module
    """
    plt.clf()
    for name in data:
        print("building " + str(name) + " ecdf...")
        xlabel = "Value"
        ylabel = "Cumulative Probability"
        title_name = name + "\n N = " + str(N) + "\n seed = " + str(seed)
        if N == 0:
            title_name = name + "\n seed = " + str(seed)
        graph = sns.displot(data[name], kind="ecdf")
        graph.set(title=title_name)
        graph.set_axis_labels(xlabel, ylabel)
        graph.savefig(name + "_ecdf.png", bbox_inches="tight")
    return


def compare_ecdf(N, RESOLUTION, seed, data):
    """
    Create graphs superposing  ecdf
    """
    plt.clf()
    BW = 0.337
    name = "compare_ecdf_"
    for e in data:
        name += e + "_"

    # subdata = {k : data[k] for k in names}
    print("graphing " + str(name) + "...")

    xlabel = "Value"
    ylabel = "Density"
    title_name = "N = " + str(N) + "\n seed = " + str(seed)
    if N == 0:
        title_name = "seed = " + str(seed)
    graph = sns.displot(data, legend=True, kind="ecdf")
    graph.set(title=title_name)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    # plt.legend()
    plt.savefig(name + ".png", bbox_inches="tight")
    return


def compare(N, RESOLUTION, seed, data):
    """
    Create graphs superposing two, already generated,
    distributivity
    """
    plt.clf()
    BW = 0.337
    name = "compare_"
    for e in data:
        name += e + "_"

    # subdata = {k : data[k] for k in names}
    print("graphing " + str(name) + "...")

    xlabel = "Value"
    ylabel = "Density"
    title_name = "N = " + str(N) + "\n seed = " + str(seed)
    if N == 0:
        title_name = "seed = " + str(seed)
    for e in data:
        graph = sns.kdeplot(
            data[e], legend=True, bw_adjust=BW, common_norm=False, label=str(e)
        )
    graph.set(title=title_name)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    graph.axes.set
    plt.legend()
    plt.savefig(name + ".png", bbox_inches="tight")
    return


def lineplot_wrap(data, xlabel, ylabel, title, iterN):
    plt.clf()
    print("graphing " + str(title) + "...")
    for e in data:
        plt.plot(iterN, data[e], marker="o")
    # graph = sns.lineplot(data, legend=True, x=iterN)
    plt.title(title)
    # plt.legend()
    plt.xscale("log")
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.savefig(title.replace(" ", "_") + ".png", bbox_inches="tight")


def lineplot_wrap_var(data, err, xlabel, ylabel, title, iterN):
    plt.clf()
    print("graphing " + str(title) + "...")
    for e in data:
        plt.plot(iterN, data[e], marker="o")
        plt.fill_between(iterN, data[e]-err, data[e]+err, alpha=0.2) 
    # graph = sns.lineplot(data, legend=True, x=iterN)
    plt.title(title)
    # plt.legend()
    plt.xscale("log")
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.savefig(title.replace(" ", "_") + ".png", bbox_inches="tight")

def lineplot_mean_alot(data, xlabel, ylabel, title, iterN):
    plt.clf()
    print("graphing " + str(title) + "...")
    for e in data:
        plt.plot(iterN, data[e], marker="o", alpha=0.05, color="red")
    plt.plot(iterN, [0] * len(iterN), color="blue", linestyle="dotted")
    plt.title(title)
    plt.xscale("log")
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.savefig(title + ".png", bbox_inches="tight")
