from datetime import datetime
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np
import scipy
import csv
import random
import multiprocessing
from graph import *

# shut up stupid warnings from not "the-most-up-to-date" libraries
import warnings

warnings.simplefilter(action="ignore", category=FutureWarning)

bold_green="\033[36;1m"
reset_style="\033[0m" 

def values(N, seed):
    """
    Return number sets of
    N values following MersenneTwister
    (Random.random()) of python
   
    print(
        "creating "
        + str(N)
        + " random values with random.random() of seed "
        #+ str(seed)
    )
    """
    random.seed()
    return np.array([random.random() for i in range(N)])

def write_stats_fields(data, name, testname):
    fields={"name" : name}
    for k in data:
        fields[k] = data[k]  
    
    #write file
    with open('stats_'+str(testname)+'.csv', 'a', newline='') as csvfile:
        datawriter = csv.DictWriter(csvfile, fieldnames=fields)
        datawriter.writerow(fields)
    return

def jobs_quadrature(f,linspaces):
    results = [] 
    name = "jQuadrature_rect"
    print("\033[34;1mStarting rectangle quadrature...\033[0m ")
    for linspace in linspaces:
        res = quadrature_riemman(f,linspace)
        results.append(res)
        print(bold_green+str(name)+str(len(linspace))+": "+reset_style+str(res)) 
        print(bold_green+str(name)+str(len(linspace))+" err :"+reset_style+str(res - I))
    results = np.array(results) - I
    lineplot_wrap({"data": results}, "Count","Error"
                  , "Error evolution of "+str(name)+" by number of points",
                  N)

    results = [] 
    name = "jQuadrature_trap"
    print("\033[34;1mStarting trapez quadrature...\033[0m ")
    for linspace in linspaces:
        res = quadrature_trapez(f,linspace)
        results.append(res)
        print(bold_green+str(name)+str(len(linspace))+": "+reset_style+str(res)) 
        print(bold_green+str(name)+str(len(linspace))+" err :"+reset_style+str(res - I))
    results = np.array(results) - I
    lineplot_wrap({"data": results}, "Count","Error"
                  ,"Error evolution of "+str(name)+" by number of points",
                  N)
    return

def quadrature_riemman(f, linspace):
    h = (linspace[-1] - linspace[0])/len(linspace)
    vf = np.vectorize(f)
    return h*np.sum(vf(linspace))

def quadrature_trapez(f, linspace):
    h = (linspace[-1] - linspace[0])/len(linspace)
    vf = np.vectorize(f)
    nlinspace = np.append((linspace[-1] - linspace[0])/ 2,linspace[1:]) 
    return h*np.sum(vf(nlinspace))

def job_MonteCarlo(f):
    entries = 10
    data = {} 
    name = "jMonteCarlo_lot"
    print("\033[34;1mStarting Monte Carlo...\033[0m ")
    vf = np.vectorize(f)
    dataresres = [] 
    for i in range(entries):
        with multiprocessing.Pool(entries) as pool:
            datares = pool.starmap(jone_Monte
                ,[(vf,name,(j+1)+i*entries,12*entries) for j in range(12)])
            pool.close()
            pool.join()
        dataresres += datares
        print(dataresres)

    for i in range(len(dataresres)):
        data[str(i)] = dataresres[i]  
    lineplot_mean_alot(data, "Count","Error"
                  ,"Error evolution of "+str(name)+" by number of points",
                  N)
    if entries==1:
        lineplot_wrap(data, "Count","Error"
                  ,"Error evolution of "+str(name)+" by number of points",
                  N)
    return

def jone_Monte(vf,name,l,entries):
    results = [] 
    for n in N:
        seed = int(datetime.now().timestamp())+l
        uniform = values(n,seed)
        uniform = uniform*(b_ - a_) + a_
        res = (b_ - a_)*np.mean(vf(uniform))
        #print(bold_green+str(name)+"_"+str(n)+": "+reset_style+str(res)) 
        #print(bold_green+str(name)+"_"+str(n)+" err :"+reset_style+str(res - I))
        results.append(res - I)
    #data[str(i)] = results
    #print(str(l)+"/"+str(entries)+"...")
    return results

def job_MonteCarlo_hist(f):
    entries = 100
    data = {} 
    name = "jMonteCarlo_hist"+str(entries*12)
    print("\033[34;1mStarting Monte Carlo Hist...\033[0m ")
    vf = np.vectorize(f)
    dataresres = [] 
    for i in range(entries):
        with multiprocessing.Pool(entries) as pool:
            datares = pool.starmap(jone_Monte
                ,[(vf,name,(j+1)+i*12,12*entries) for j in range(12)])
            pool.close()
            pool.join()
        dataresres+=datares
        print(str((i+1)*12)+"/"+str(entries*12)+"...")

    data = {"0": dataresres}
    print(data)
    hist_distributivity_graph(N, 25, 0,data,"Result values of"+str(name)+".")
    return 


def job_MonteCarlo_batch(f):
    entries = 10
    data = {} 
    name = "jMonteCarlo_batch"+str(entries*12)
    print("\033[34;1mStarting Monte Carlo...\033[0m ")
    vf = np.vectorize(f)
    dataresres = [] 
    for i in range(entries):
        with multiprocessing.Pool(entries) as pool:
            datares = pool.starmap(jone_Monte
                ,[(vf,name,(j+1)+i*entries,12*entries) for j in range(12)])
            pool.close()
            pool.join()
        dataresres += datares

    data = {"0": np.mean(dataresres, axis=0)}
    var = np.mean(np.square(dataresres - data[0]), axis=0)
    err = 1.96*np.sqrt(var)*(1/np.sqrt(len(var)))
    lineplot_wrap_var(data, err, "Count","Error"
                  ,"Error evolution of "+str(name)+" by number of points",
                  N)
    return 

if __name__ == "__main__":
    sns.set_theme(style="whitegrid", palette="rocket")
    N = [10000] # 10**6 random values
    Ns = 12
    RESOLUTION = 25
    Np = [1000,10000,100000,1000000,10000000]
    b_ = .9
    a_ = .1
    
    seed = int(datetime.now().timestamp())
    print("\033[36;1mExpected Integral result: \033[0m",end="")

    def f(x):
        return .5*np.exp(np.arcsin(x))*(x/np.sqrt(1 - x**2))
    linspaces=[np.linspace(a_,b_,n_p,endpoint=False) for n_p in Np]
 
    jobs = [  
        #[jobs_quadrature,[f,linspaces]],
        #[job_MonteCarlo,[f]] 
        [job_MonteCarlo_hist,[f]]  
    ]

    I =  .25*(np.exp(np.arcsin(.9))*(.9 - np.sqrt(0.19)) + np.exp(np.arcsin(.1))*(np.sqrt(0.99) - 0.1))
    
    print(I)

    for j in jobs:
        multiprocessing.Process(target=j[0],args=j[1]).start()
