import graph
from datetime import datetime
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np
import scipy
import csv
import random
import multiprocessing

# shut up stupid warnings from not "the-most-up-to-date" libraries
import warnings

warnings.simplefilter(action="ignore", category=FutureWarning)


def values(N, seed):
    """
    Return number sets of
    N values following MersenneTwister
    (Random.random()) of python
    """
    print(
        "creating "
        + str(N)
        + " random values with random.random() of seed "
        + str(seed)
    )
    random.seed(seed)
    return np.array([random.random() for i in range(N)])

def Marsaglia_pair(N, seed):
    """
    Return a pair of two N random uniform values W1 W2
    in [-1;1] 
    respecting :
    for each x,y W1,W2
        0 <= x**x + y**y < 1
    """
    print(
        "creating "
        + str(N)
        + " random pair values for Marsaglia with random.random() of seed "
        + str(seed)
    )
    W0 = np.zeros(N)
    W1 = np.zeros(N)
    random.seed(seed)
    i=0
    while i < N:
        w0 = 2*random.random() - 1
        w1 = 2*random.random() - 1
        while (w0**2 + w1**2 >= 1):
            w0 = 2*random.random() - 1
            w1 = 2*random.random() - 1

        W0[i] = w0
        W1[i] = w1
        i+=1
    return [W0, W1] 

def reject_val(N, seed, Inv, c, g, f):
    """
    Return N random values
    following reject method on c*g(X)*rand < f(X)
    """
    print(
        "creating "
        + str(N)
        + " random pair values with reject method with random.random() of seed "
        + str(seed)
    )
    i=0
    random.seed(seed)
    res = np.zeros(N)
    while i < N:
        u = Inv(random.random())
        while (c*g(u)*random.random() > f(u)):
            u = Inv(random.random())
        res[i] = u
        i += 1
    return res

def append_values(N, seed, where):
    where.append(values(N, seed))
    return


def more_values(N, seed, number):
    with multiprocessing.Pool(number) as pool:
        res = pool.starmap(values, [(N, seed + i) for i in range(number)])
        pool.close()
        pool.join()
        # res.append(values(N, seed+i))
    #print(res)
    return np.array(res)


def job_erfinv(uniform):
    name="erfinv"
    print("\033[32;1mSarting erfinv job\033[0m")
    data_erfinv = NormCenterErfInv(uniform)
    graph.hist_distributivity_graph(N, RESOLUTION
                                    , seed, {"erfinv": data_erfinv})
    do_stats_tests(data_erfinv, name)
    return data_erfinv


def job_CentralLimitTheo(uniforms):
    name="CentralLimitTheorem"
    print("\033[32;1mSarting Central Limit job\033[0m")
    n = len(uniforms)
    res = np.divide(np.sum(uniforms, axis=0) - (n * 0.5), np.sqrt(n / 12))
    #print(res)
    graph.hist_distributivity_graph(N, RESOLUTION, 0
                                    , {"CentralLimitTheorem" + str(n): res})
    do_stats_tests(res,name)
    return

def NormCenterErfInv(uniform):
    return np.sqrt(2) * scipy.special.erfinv(2 * uniform - 1)

def job_BoxMuller(uniforms):
    name="BoxMuller"
    print("\033[32;1mSarting BoxMuller job\033[0m")
    if (len(uniforms) != 2):
        raise ValueError("need two uniforms values exactly.")
    U0 = uniforms[0]
    U1 = uniforms[1]
    Y0 = np.sqrt(-2*np.log(U0)) * np.cos(2*np.pi*U1)
    Y1 = np.sqrt(-2*np.log(U1)) * np.cos(2*np.pi*U0)
    graph.hist_distributivity_graph(N, RESOLUTION, 0
                                    , {"BoxMuller1" : Y0})
    graph.hist_distributivity_graph(N, RESOLUTION, 0
                                    , {"BoxMuller2" : Y1})
    do_stats_tests(Y0, name+"1")
    do_stats_tests(Y1, name+"2")
    return

def job_Marsagalia():
    name = "Marsaglia"
    print("\033[32;1mSarting Marsaglia job\033[0m")
    pair = Marsaglia_pair(N,seed)
    W0 = pair[0]
    W1 = pair[1] 
    squared = np.square(W1)+np.square(W0)
    Y0 = np.sqrt(-2*np.log(squared)) * np.divide(W1,np.sqrt(squared))
    Y1 = np.sqrt(-2*np.log(squared)) * np.divide(W0,np.sqrt(squared))
    graph.hist_distributivity_graph(N, RESOLUTION, 0
                                    , {"Marsaglia1" : Y0})
    graph.hist_distributivity_graph(N, RESOLUTION, 0
                                    , {"Marsaglia2" : Y1})

    do_stats_tests(Y0, name+"1")
    do_stats_tests(Y1, name+"2")
    return

def GINVcauchy(x):
        return np.tan(np.pi*(x - (1/2)))

def gcauchy(x):
        return (1/np.pi) * (1/(1+np.square(x)))

cCauchy = np.sqrt(2*np.pi / np.e)

def job_rejectCauchy():
    name = "rejectCauchy"
    print("\033[32;1mStarting Cauchy reject job\033[0m")
    number = 10
    with multiprocessing.Pool(number) as pool:
        res = pool.starmap(reject_val
                ,[(int(N/10), seed+i, GINVcauchy, cCauchy, gcauchy 
                   ,scipy.stats.norm.pdf) for i in range(number)])
        pool.close()
        pool.join()
    fres = np.array(res).flatten()
    graph.hist_distributivity_graph(N, RESOLUTION, 0
                                    , {name : fres})
    do_stats_tests(fres, name)

def job_MultiDim(two_uniform):
    name = "MultiDim"
    print("\033[32;1mStarting "+str(name)+" job\033[0m")

    cov = np.array([[1,0.5],[0.5,1]]) 
    res = np.array([np.zeros(N),np.zeros(N)])
    U0 = two_uniform[0]
    U1 = two_uniform[1] 
    res[0] = NormCenterErfInv(U0) 
    res[1] = NormCenterErfInv(U1)
    
    print(res)
    
    evalues, evectors = np.linalg.eig(cov)
    D = np.diag(np.flip(evalues))
    U = np.transpose(evectors)
    Uinv = np.linalg.inv(U)
    X = np.matmul(U, np.matmul(D,res))
    
    graph.kde_2distributivity_graph(N*2,RESOLUTION, 0, X, cov, name)
    return

def job_MultiDimBox(two_uniform):
    name = "MultiDimE"
    print("\033[32;1mStarting "+str(name)+" job\033[0m")

    cov = np.array([[1,0],[0,1]]) 
    res = np.array([np.zeros(N),np.zeros(N)])
    U0 = two_uniform[0]
    U1 = two_uniform[1] 
    res[0] = np.sqrt(-2*np.log(U0)) * np.cos(2*np.pi*U1)
    res[1] = np.sqrt(-2*np.log(U1)) * np.cos(2*np.pi*U0)
  
    print(scipy.stats.pearsonr(res[0],res[1]))
     
    #res[0] = NormCenterErfInv(U0)
    #res[1] = NormCenterErfInv(U1)

    #print(scipy.stats.pearsonr(res[0],res[1]))
    graph.kde_2distributivity_graph(N*2,RESOLUTION, 0, res, cov, name)
    return

def write_stats_fields(data, name, testname):
    fields={"name" : name}
    for k in data:
        fields[k] = data[k]  
    
    #write file
    with open('stats_'+str(testname)+'.csv', 'a', newline='') as csvfile:
        datawriter = csv.DictWriter(csvfile, fieldnames=fields)
        datawriter.writerow(fields)
    return

def do_stats_tests(normals, name):
    shap = scipy.stats.shapiro(normals)
    write_stats_fields({"W" : shap[0], "pvalue" : shap[1]}, name
        , "shapiroWilk")
    #Lilli = statsmodels.stats.diagnostic.lilliefors(normals)
    #write_stats_fields({"Kolmogorov-Smirnov" : Lilli[0], "pvalue" : Lilli[1]}
    #                   , name, "LillieFors")

    ander = scipy.stats.anderson(normals)
    write_stats_fields({"test" : ander[0]}, name, "AndersonDarling")
    ago = scipy.stats.normaltest(normals)
    write_stats_fields({"test" : ago[0], "pvalue" : ago[1]}, name, "Agostino")
    return


if __name__ == "__main__":
    sns.set_theme(style="whitegrid", palette="rocket")
    N = 100000 # 10**6 random values
    Ns = 12
    RESOLUTION = 25
    seed = int(datetime.now().timestamp())
    data = more_values(N, seed, Ns)
    #PROCESSING################################################################
#    p = multiprocessing.Process(target=job_erfinv, args=[data[0]])
#    p.start()
#    p = multiprocessing.Process(target=job_CentralLimitTheo, args=[data])
#    p.start()
#    p = multiprocessing.Process(target=job_BoxMuller, args=[data[0:2]])
#    p.start()
#    p = multiprocessing.Process(target=job_Marsagalia, args=[])
#    p.start()
#    p = multiprocessing.Process(target=job_rejectCauchy, args=[])
#    p.start()
    #multiprocessing.Process(target=job_MultiDim,args=[data[0:2]]).start()
    multiprocessing.Process(target=job_MultiDimBox,args=[data[0:2]]).start()
