#Uses MPI to dispatch random number generation on multiple nodes.
#files are saved into .npy files which can be loaded later with numpy
from mpi4py import MPI
from generators import *
from datetime import datetime

comm = MPI.COMM_WORLD
size = comm.Get_size() # we suppose > 1
rank = comm.Get_rank()

#Wrapper for outside call
def MPIBatchRandomNormalized(generator,elmPerNode):
    name = type(generator).__name__
    N = elmPerNode

    if rank == 0:
        print("processing "+name+"...")

    #set seed with rank offset
    generator.seed(rank + datetime.now().timestamp())
    randomNumbers = generator.batchRandomNormalized(N)
    
    #receives all the results. 
    gathered = comm.gather(data, root=0)
    if rank==0:
        data = np.array()
        for l in gathered:
            data += np.array(l)
        print("Saving to "+name+".npy ...")
        with open(name+".npy", "wb") as f:
            np.save(f, data)

if __name__ == "__main__":
    N = 1000000
    generators = [ParkMiller(), KnuthLewis(), Marsaglia()
                  , LavauxJenssens(), Haynes(), MitchelMoore()
                  , MersenneTwister(), BlumBlumShub()]
    
    for e in generators:
        MPIBatchRandomNormalized(e,N)
