from datetime import datetime
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np
import scipy
import csv
import random
import multiprocessing
from graph import *

# shut up stupid warnings from not "the-most-up-to-date" libraries
import warnings

warnings.simplefilter(action="ignore", category=FutureWarning)

bold_green="\033[36;1m"
reset_style="\033[0m" 

def values(N, seed):
    """
    Return number sets of
    N values following MersenneTwister
    (Random.random()) of python
    """
    print(
        "creating "
        + str(N)
        + " random values with random.random() of seed "
        + str(seed)
    )
    random.seed()
    return np.array([random.random() for i in range(N)])

def write_stats_fields(data, name, testname):
    fields={"name" : name}
    for k in data:
        fields[k] = data[k]  
    
    #write file
    with open('stats_'+str(testname)+'.csv', 'a', newline='') as csvfile:
        datawriter = csv.DictWriter(csvfile, fieldnames=fields)
        datawriter.writerow(fields)
    return

def venDerCorput(n, base):
    fact = 1/base
    res = 0

    while n > 0:
        res += (n % base) * fact
        n //= base
        fact /= base
    
    return res

def venDerCorputAll(N, base):
    res = np.arange(1,N)
    vfunc = np.vectorize(venDerCorput)
    return vfunc(res, base)

def j_testVenDerMany():
    for N in [5, 10, 50, 100]:
        for base in [10,2,19,41]:
            name = "VenDerCorput with "+str(N)+" values and base "+str(base)+" ("+(str(N**2))+" points)."
            multiprocessing.Process(target=j_testVenDerOne
                                    , args=[N,base,name]).start()
    return

def j_testVenDerManyD():
    bases = [3, 5, 19, 41] 
    for N in [10, 100, 1000, 10000]:
        for i in range(len(bases)):
            base = (bases[i], bases[(i+1) % len(bases)])
            name = "VenDerCorput with "+str(N)+" values and base "+str(base)
            multiprocessing.Process(target=j_testVenDerTwo
                                    , args=[N,base,name]).start()
    return

def j_testVenDerOne(N,base,name):
    print("\033[92mProcessing venDerCorput with "
            +str(N)+","+str(base)+"\033[0m" )
    r = venDerCorputAll(N,base)
    graph_scatter_same(r,r,name)
    return

def j_testVenDerTwo(N,base,name):
    print("\033[92mProcessing venDerCorput with "
            +str(N)+","+str(base)+"\033[0m" )
    rx = venDerCorputAll(N,base[0])
    ry = venDerCorputAll(N,base[1])
    graph_scatter(rx,ry,name,base)
    return

def f(x):
    return .5*np.exp(np.arcsin(x))*(x/np.sqrt(1 - x**2))

def j_MonteCarlo():
    b_ = .9
    a_ = .1
    linspaces=[np.linspace(a_,b_,n_p,endpoint=False) 
               for n_p in 
               [1000,10000,100000,1000000,10000000]]
    entries = 10
    data = {} 
    name = "jMonteCarlo_lot"
    print("\033[34;1mStarting Monte Carlo...\033[0m ")
    vf = np.vectorize(f)
    dataresres = [] 
    values = venDerCorputAll(100000000,3)
    for i in range(entries):
        with multiprocessing.Pool(entries) as pool:
            datares = pool.starmap(jone_Monte
                ,[(vf,name,(j+1)+i*entries,12*entries,values) 
                  for j in range(12)])
            pool.close()
            pool.join()
        dataresres += datares
    for i in range(len(dataresres)):
        data[str(i)] = dataresres[i]  
    lineplot_mean_alot(data, "Count","Error"
                  ,"Error evolution of "+str(name)+" by number of points",
                  [1000,10000,100000,1000000,10000000])
    return

def jone_Monte(vf,name,l,entries,values):
    print("\t\033[34mStarting one sub job...\033[0m")
    b_ = .9
    a_ = .1
    results = []
    I =  .25*(np.exp(np.arcsin(.9))*(.9 - np.sqrt(0.19)) + np.exp(np.arcsin(.1))*(np.sqrt(0.99) - 0.1))

    for n in [1000,10000,100000,1000000,10000000]:
        np.random.seed(int(datetime.now().timestamp())+l)
        uniform = np.random.choice(values, n)
        uniform = uniform*(b_ - a_) + a_
        res = (b_ - a_)*np.mean(vf(uniform))
        
        results.append(res - I)
    return results

if __name__ == "__main__":
    sns.set_theme(style="whitegrid", palette="rocket")

    seed = int(datetime.now().timestamp())
 
    jobs = [  
        #[j_testVenDerMany,[]],
        [j_testVenDerManyD,[]],
        #[j_MonteCarlo,[]] 
    ]


    for j in jobs:
        multiprocessing.Process(target=j[0],args=j[1]).start()
